/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name$
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.core;

import net.sf.jguard.core.filters.Filterable;
import net.sf.jguard.core.filters.FilterChainImpl;
import net.sf.jguard.core.filters.Filter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;



import net.sf.jguard.core.authentication.AccessContext;
import net.sf.jguard.core.authentication.manager.AuthenticationManager;
import net.sf.jguard.core.authentication.manager.AuthenticationManagerFactory;
import net.sf.jguard.core.filters.PolicyEnforcementPointFilter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * Policy Enforcement Point.
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 * @since 1.1
 */
public class PolicyEnforcementPoint implements Cloneable{

    private static final Logger logger = LoggerFactory.getLogger(PolicyEnforcementPoint.class.getName());
    
    private List<Filter> filters;

    private PolicyEnforcementPoint(){
        
    }

    public PolicyEnforcementPoint(String authenticationBindingsFactoryImpl,Map<PolicyEnforcementPointOptions,String> options){
       
       PolicyEnforcementPointFilter pepFilter = new PolicyEnforcementPointFilter(authenticationBindingsFactoryImpl,options);
       filters = new ArrayList<Filter>();
       AuthenticationManager authenticationManager = AuthenticationManagerFactory.getAuthenticationManager();
       if(authenticationManager!=null && authenticationManager instanceof Filterable){
           logger.debug("adding authenticationManager filter");
           addFilter(((Filterable)authenticationManager).getFilter());
       }
       addFilter(pepFilter);
    }
    
         
    /**
     * check if user is authenticated, and check its access rights.
     * @param context
     */
	public void process(AccessContext context){
		try{
                    FilterChainImpl filterChain = new FilterChainImpl(filters);
                    filterChain.doFilter(context);
                }catch(Throwable t){
                    logger.error(t.getMessage(),t);
                    throw new IllegalStateException(" an exception has occured ");
                }

	}
 
   public void addFilter(Filter filter){
        filters.add(filter);
   }

   public void addFilter(int index,Filter filter){
        filters.add(index,filter);
   }

   public Object clone(){
       PolicyEnforcementPoint p = new PolicyEnforcementPoint();
       p.filters = new ArrayList<Filter>(filters);
       return p;
   }
}
