/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name$
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.core.authentication;

import java.util.Locale;


import java.util.logging.Level;
import javax.security.auth.Subject;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.login.Configuration;
import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;

import net.sf.jguard.core.principals.UserPrincipal;
import net.sf.jguard.core.util.ThrowableUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * Authentication wrapper around the {@link Subject}, which permits to infer
 *  on the Subject's LifeCycle (login, logout). 
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 */
public class AuthenticationUtils {

	private LoginContext loginContext = null;
	private Subject subject = null;           
    private AuthenticationStatus status;
	private boolean loggedOut = false;
	private boolean local = false;
        private static final Logger logger = LoggerFactory.getLogger(AuthenticationUtils.class.getName());
	private Configuration configuration = null;
	
	public AuthenticationUtils(){
		super();
	}
	
	
	public AuthenticationUtils(Configuration config){
		super();
		configuration = config;
		local= true;
	}
	
	/**
	 * authenticate user against the application's configuration.
	 * @param applicationName
	 * @param cbh 
	 * @throws LoginException raised if authentication failed
	 */
	public void login( String applicationName, CallbackHandler cbh) throws LoginException {
                if(local){
                    loginContext = new LoginContext(applicationName,new Subject(),cbh,configuration);
                }else{
                    loginContext = new LoginContext(applicationName,cbh);
                }
                try{
                    loginContext.login();
                }catch(LoginException le){
                    Throwable localizedThrowable = ThrowableUtils.localizeThrowable(le,Locale.getDefault());
                    throw (LoginException)localizedThrowable;
                }

                subject = loginContext.getSubject();
           
                if(subject != null){
                        // used in ABAC permissions
                        UserPrincipal userPrincipal = new UserPrincipal(subject);
                        subject.getPrincipals().add(userPrincipal);
                }
	}


	
	
	 /**
     * retrieve the subject from the loginContext.
     * @return authenticated Subject, otherwise <strong>null</strong>.
     */
    public Subject getSubject(){
    	return subject;
    }
    
    /**
     * logout the user with the related LoginContext.
     */
    public  void logout() {
      if(loggedOut==false){
    	 try {
    		
                if(loginContext!=null){
                loginContext.logout();
                loggedOut = true;
                }else{
                        logger.debug(" user is not logged, so we don't logout him ");
                }
    		
         } catch (LoginException e) {
             logger.debug(" error raised when the user logout "+e.getMessage(),e);
         }
      }
    }

	public boolean isLocal() {
		return local;
	}

    /**
     * @return the status
     */
    public AuthenticationStatus getStatus() {
        return status;
    }

    /**
     * @param status the status to set
     */
    public void setStatus(AuthenticationStatus status) {
        this.status = status;
    }
	
	
}
