/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name$
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.core.authentication.bindings;


import net.sf.jguard.core.CoreConstants;
import net.sf.jguard.core.authentication.AccessContext;
import net.sf.jguard.core.authentication.AuthenticationUtils;

import javax.security.auth.login.Configuration;


public abstract class AbstractAuthenticationBindings implements AuthenticationBindings {


    protected String authScheme = null;
    private String scope;
    protected AccessContext context;
    protected AuthenticationBindingsFactory factory;

    public AbstractAuthenticationBindings(AuthenticationBindingsFactory factory, String authenticationScope, AccessContext context) {
        super();
        if (CoreConstants.JVM_SCOPE.equalsIgnoreCase(authenticationScope)) {
            scope = CoreConstants.JVM_SCOPE;
        } else {
            scope = CoreConstants.LOCAL_SCOPE;
        }
        this.context = context;
        this.factory = factory;

    }


    /**
     * grab the {@link AuthenticationUtils} from the session, or create a new one if no one exist.
     *
     * @return AuthenticationUtils
     */
    public AuthenticationUtils getAuthenticationUtils() {
        AuthenticationUtils authenticationUtils = null;
        if (this instanceof StatefulAuthenticationBindings) {
            authenticationUtils = (AuthenticationUtils) ((StatefulAuthenticationBindings) this).getSessionAttribute(CoreConstants.AUTHN_UTILS);
            //auth.getSubject() can return  null if the user in session has been deleted by authenticationManager
        }


        if (authenticationUtils == null) {
            Configuration configuration = (Configuration) this.getApplicationAttribute(CoreConstants.JGUARD_CONFIGURATION);
            if (CoreConstants.LOCAL_SCOPE.equals(scope) && configuration != null) {
                authenticationUtils = new AuthenticationUtils(configuration);
            } else {
                authenticationUtils = new AuthenticationUtils();
            }
            if (this instanceof StatefulAuthenticationBindings) {
                ((StatefulAuthenticationBindings) this).setSessionAttribute(CoreConstants.AUTHN_UTILS, authenticationUtils);
            }
        }
        return authenticationUtils;
    }


    public AccessContext getContext() {
        return context;
    }

    public AuthenticationBindingsFactory getAuthenticationBindingsFactory() {
        return factory;
    }

    /**
     * @return the local
     */
    public String getScope() {
        return scope;
    }

}
