/*
	jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name$
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/

package net.sf.jguard.core.authentication.bindings;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import net.sf.jguard.core.authentication.schemes.AuthenticationSchemeHandler;
import net.sf.jguard.core.util.XMLUtils;
import org.dom4j.Document;
import org.dom4j.Element;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * subclasses must have a String constructor.
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 */
public abstract class AbstractAuthenticationBindingsFactory implements AuthenticationBindingsFactory{

    private static Logger logger = LoggerFactory.getLogger(AbstractAuthenticationBindingsFactory.class.getName());
 
    protected String scope;
    public AbstractAuthenticationBindingsFactory(String authenticationScope){
        this.scope = authenticationScope;
    }
    protected List<AuthenticationSchemeHandler> authSchemeHandlers;
    
    
    public void init(String location) {
         if(location==null||"".equals(location)){
             throw new IllegalArgumentException(" jGuardFilter.xml location is null ");
         }
         this.authSchemeHandlers = loadFilterConfiguration(location);
         
    }
    
     public List<AuthenticationSchemeHandler> getAuthenticationSchemeHandlers() {
		return authSchemeHandlers;
     }		
          
	   /** load configuration from an XML file.
        * @param configurationLocation
        * @return Map containing filter configuration
        */
        public List<AuthenticationSchemeHandler> loadFilterConfiguration(String configurationLocation){
                Document doc = XMLUtils.read(configurationLocation);

                Element callbackHandlerElement = doc.getRootElement();
                if(doc==null ||callbackHandlerElement == null){
                    throw new IllegalArgumentException(" xml file at this location:"+configurationLocation+" is not found or cannot be read ");
                }
                List<AuthenticationSchemeHandler> authenticationSchemeHandlers = new ArrayList<AuthenticationSchemeHandler>(1); 
                
                Iterator it = callbackHandlerElement.elementIterator("authenticationSchemeHandler");
                while(it.hasNext()){
                    Element authenticationSchemeHandlerElement = (Element)it.next();
                    String className = authenticationSchemeHandlerElement.attributeValue("className");
                    Iterator itParameters = authenticationSchemeHandlerElement.elementIterator("parameter");
                    Map<String,String> parameters = new HashMap<String,String>(2);
                    while(itParameters.hasNext()){
                        Element parameterElement = (Element)itParameters.next();
                        String key = parameterElement.attributeValue("key");
                        String value = parameterElement.attributeValue("value");
                        parameters.put(key, value);
                    }
                    Class<AuthenticationSchemeHandler> authenticationSchemeHandlerClass = null;
                    try {
                        authenticationSchemeHandlerClass = (Class<AuthenticationSchemeHandler>) Thread.currentThread().getContextClassLoader().loadClass(className);
                    } catch (ClassNotFoundException ex) {
                        logger.error("authenticationSchemeHandler className cannot be found "+ ex.getMessage(), ex);
                        continue;
                    }
                    AuthenticationSchemeHandler authenticationSchemeHandler = null;
                    try {
                        Class[] types = new Class[]{Map.class,AuthenticationBindingsFactory.class};
                        Constructor constructor = authenticationSchemeHandlerClass.getConstructor(types);
                        
                        authenticationSchemeHandler = (AuthenticationSchemeHandler) constructor.newInstance(parameters,this);
                        authenticationSchemeHandlers.add(authenticationSchemeHandler);
                    }catch (InstantiationException ex) {
                        logger.error("authenticationSchemeHandler cannot be instantiated "+ex.getMessage(), ex);
                        continue;
                    } catch (IllegalAccessException ex) {
                        logger.error("authenticationSchemeHandler class cannot be accessed"+ex.getMessage(), ex);
                        continue;
                    }catch (InvocationTargetException ex) {
                        logger.error(ex.getMessage(), ex);
                    } catch (NoSuchMethodException ex) {
                                logger.error(ex.getMessage(), ex);
                    } catch (SecurityException ex) {
                        logger.error(ex.getMessage(), ex);
                    }
                        
                }
                if(authenticationSchemeHandlers.size()==0){
                    throw new IllegalStateException(" the configuration File "+configurationLocation+" does not contains any authenticationSchemeHandler ");
                }
                return authenticationSchemeHandlers;
        }

        /**
         * instantiate an AuthenticationBindingsFactory with its AuthenticationScope one Argument.
         * @param implementation
         * @param authenticationScope
         * @return
         * @throws java.lang.Exception
         */
        public static AuthenticationBindingsFactory getAuthenticationBindingsFactory(String implementation,String authenticationScope) throws Exception{
            AuthenticationBindingsFactory factory = null;
            Class factoryClass = Thread.currentThread().getContextClassLoader().loadClass(implementation);
            factory = (AuthenticationBindingsFactory) factoryClass.getConstructor(String.class).newInstance(authenticationScope);
            return factory;
        }
}
