/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name$
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/


package net.sf.jguard.core.authentication.bindings;


import net.sf.jguard.core.PolicyEnforcementPoint;
import net.sf.jguard.core.authentication.AccessContext;
import net.sf.jguard.core.authentication.AuthenticationServicePoint;
import net.sf.jguard.core.authentication.AuthenticationUtils;

import javax.security.auth.callback.CallbackHandler;


/**
 * Authentication bindings with the underlying protocol and server technology
 * used by the {@link PolicyEnforcementPoint}, <b>specific to an AccessContext</b>.
 * Note that implementation of this interface <strong>DOES NOT</strong>
 * authenticate any entity.
 *
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 * @see PolicyEnforcementPoint
 * @see AuthenticationServicePoint
 * @since 1.1
 */
public interface AuthenticationBindings {


    //actions
    public void process();

    /**
     * return a CallbackHandler bounded to the current AccessContext.
     *
     * @return
     */
    public CallbackHandler getCallbackHandler();


    public AccessContext getContext();

    /**
     * extract from the AccessContext the AuthenticationUtils.
     * note that each AuthenticationUtils instance is related to a Subject,
     * so different AuthenticationUtils coexist.It can be done because
     * AccessContext is different among users, although if they share <b>the same</b> AuthenticationBindings
     * instance: it implies that AuthenticationBindings is only the method to extract from
     * AccessContext the AuthenticationUtils (and its underlying Subject).
     *
     * @return
     */
    public AuthenticationUtils getAuthenticationUtils();

    //request specific method
    public void setRequestAttribute(String key, Object value);

    public Object getRequestAttribute(String key);

    public void removeRequestAttribute(String key);


    //application specific method
    public void setApplicationAttribute(String key, Object value);

    public Object getApplicationAttribute(String key);

    public void removeApplicationAttribute(String key);

    /**
     * parameter defined for initialization purpose, reachable
     * at an application scope.
     *
     * @param key
     * @return value as a String
     */
    public String getInitApplicationAttribute(String key);

    public AuthenticationBindingsFactory getAuthenticationBindingsFactory();


    public String getScope();
}
