/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name$
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.core.authentication.bindings;

import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;

import net.sf.jguard.core.authentication.callbacks.HookCallbackHandler;
import net.sf.jguard.core.authentication.schemes.AuthenticationSchemeHandler;

/**
 * encapsulate callbacks grabbed with the provided {@link AuthenticationBinding} {@link CallbackHandler}'s
 * implementation, by some provided credentials. it permits to act as the user which own these credentials.
 * It wraps AuthenticationBindingsFactory and CallbackHandler.
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 */
public class ImpersonationAuthenticationBindings extends AuthenticationBindingsWrapper {
    private ImpersonationAuthenticationBindingsFactory factory = null;
    private List<Callback> callbacks = null;
    private Map requestAttributes = null;
    private Map applicationAttributes = null;
        /**
         * 
         * @param authNBindings wrapped AuthenticationBindings
         * @param cbks
         */
	public ImpersonationAuthenticationBindings(AuthenticationBindings authNBindings,List<Callback> cbks){
		super(authNBindings);
		this.callbacks = cbks;
        factory = new ImpersonationAuthenticationBindingsFactory(authNBindings.getAuthenticationBindingsFactory());
        requestAttributes = new HashMap();
        applicationAttributes = new HashMap();
        
	}

    @Override
    public Object getRequestAttribute(String key){
        return requestAttributes.get(key);
    }

    @Override
    public void setRequestAttribute(String key,Object value){
        requestAttributes.put(key, value);
    }

    @Override
    public void removeRequestAttribute(String key){
        requestAttributes.remove(key);
    }


   @Override
   public void setApplicationAttribute(String key,Object value){
        applicationAttributes.put(key, value);
   }
   @Override
   public Object getApplicationAttribute(String key){
       Object value = applicationAttributes.get(key);
       if(value==null){
        return authNBindings.getApplicationAttribute(key);
       }else{
           return value;
       }
   }
   @Override
   public void removeApplicationAttribute(String key){
        applicationAttributes.remove(key);
   }
   


    @Override
	public CallbackHandler getCallbackHandler(){
		return new HookCallbackHandler(callbacks);
	}

	public List<Callback> getCallbacks() {
		return callbacks;
	}

	public void setCallbacks(List<Callback> callbacks) {
		this.callbacks = callbacks;
	}
        
    @Override
    public AuthenticationBindingsFactory getAuthenticationBindingsFactory() {
            return factory;
    }


    public void addAuthenticationSchemeHandlerToFactory(Collection<AuthenticationSchemeHandler> handlers){
        factory.addAuthenticationSchemeHandler(handlers);
    }

    
}
