/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name:  $
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.core.authentication.manager;

import java.security.NoSuchAlgorithmException;
import java.util.Map;

import net.sf.jguard.core.CoreConstants;
import net.sf.jguard.core.PolicyEnforcementPointOptions;
import net.sf.jguard.core.authentication.AuthenticationException;
import net.sf.jguard.core.authentication.configuration.JGuardConfiguration;
import net.sf.jguard.core.util.CryptUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;



/**
 * utility class used to <strong>load</strong> Authentication configuration
 * and to create and register an {@link AuthenticationManager}. 
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 * @author <a href="mailto:vberetti@users.sourceforge.net">Vincent Beretti</a>
 */
public class AuthenticationHelper {

	private static final Logger logger = LoggerFactory.getLogger(AuthenticationHelper.class.getName());


	/**
	 * create an {@link AuthenticationManager} and register it.
	 * @param authenticationOptions
	 * @param applicationName
	 * @return authenticationManager created
	 * @throws IllegalArgumentException
	 */
	public static AuthenticationManager initAuthenticationManager(JGuardConfiguration jGuardConf,Map authenticationOptions, String applicationName) throws IllegalArgumentException {

		if (logger.isDebugEnabled()) {
			logger.debug("authenticationOptions="+ authenticationOptions);
		}

		initCryptOptions(authenticationOptions);
		
		//register the authentication part
		
		Map authManagerOptions = (Map)authenticationOptions.get(CoreConstants.AUTHENTICATION_MANAGER_OPTIONS);
		authManagerOptions.put(PolicyEnforcementPointOptions.APPLICATION_NAME.getLabel(), applicationName);

		try {
			Class AuthenticationManagerSubclass = Thread.currentThread().getContextClassLoader().loadClass((String)authenticationOptions.get(CoreConstants.AUTHENTICATION_MANAGER));
			AuthenticationManagerFactory.setAuthenticationManager(AuthenticationManagerFactory.createAuthenticationManager(AuthenticationManagerSubclass, authManagerOptions));
		}catch (ClassNotFoundException e) {
			logger.error("ClassNotFoundException authenticationManager initialization failed"+e.getMessage());
		} catch (AuthenticationException e) {
			logger.error("AuthenticationException authenticationManager initialization failed"+e.getMessage());
		} 

		return AuthenticationManagerFactory.getAuthenticationManager();
	}


	private static void initCryptOptions(Map authenticationOptions) {
		String salt = ((String)authenticationOptions.get(CoreConstants.SALT));
		String digestAlgorithm = ((String)authenticationOptions.get(CoreConstants.DIGEST_ALGORITHM));
		if(digestAlgorithm!= null && !"".equals(digestAlgorithm)){
			try {
				CryptUtils.setDigestAlgorithm(digestAlgorithm);
			} catch (NoSuchAlgorithmException e) {
				logger.error(e.getMessage());
				throw new IllegalArgumentException(e.getMessage());
			}

			if(	salt!=null && !salt.equals("")){
		        boolean setSaltSuccess = CryptUtils.setSalt(salt.toCharArray());
		        if(!setSaltSuccess){
		        	logger.warn(" salt for message digest has not been set ");
		        }
			}
		}
	}


	
}
