/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name$
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.core.authentication.schemes;

import net.sf.jguard.core.authentication.AccessContext;
import net.sf.jguard.core.authentication.AuthenticationException;
import net.sf.jguard.core.authentication.bindings.AuthenticationBindingsFactory;
import net.sf.jguard.core.authorization.permissions.JGPositivePermissionCollection;

import javax.security.auth.callback.Callback;
import javax.security.auth.callback.NameCallback;
import javax.security.auth.callback.PasswordCallback;
import javax.security.auth.callback.UnsupportedCallbackException;
import java.security.Permission;
import java.security.PermissionCollection;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * HTTP FORM which requires a login (NameCallback) and a password (PasswordCallback).
 *
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 * @see NameCallback
 * @see PasswordCallback
 * @since 1.1
 */
public abstract class FormSchemeHandler implements StatefulAuthenticationSchemeHandler {
    private ArrayList<Class> callbackTypes = null;
    protected AuthenticationBindingsFactory factory;
    protected PermissionCollection grantedPermissions;
    protected Permission logoffPermission;
    protected Permission logonPermission;
    protected Permission logonProcessPermission;
    protected boolean goToLastAccessDeniedUriOnSuccess = true;
    public static final String LOGIN = "login";
    public static final String PASSWORD = "password";

    public FormSchemeHandler(Map<String, String> parameters, AuthenticationBindingsFactory factory) {
        callbackTypes = new ArrayList<Class>();
        callbackTypes.add(NameCallback.class);
        callbackTypes.add(PasswordCallback.class);
        this.factory = factory;
        this.goToLastAccessDeniedUriOnSuccess = Boolean.parseBoolean((String) parameters.get("goToLastAccessDeniedUriOnSuccess"));
        initSettings(parameters);

        grantedPermissions = new JGPositivePermissionCollection();
        grantedPermissions.add(logonPermission);
        grantedPermissions.add(logoffPermission);
        grantedPermissions.add(logonProcessPermission);
    }

    protected abstract void initSettings(Map<String, String> parameters) throws IllegalArgumentException;

    public String getName() {
        return "FORM";
    }

    public PermissionCollection getGrantedPermissions() {
        return grantedPermissions;
    }

    public Collection<Class> getCallbackTypes() {
        return callbackTypes;
    }

    public boolean answerToChallenge(AccessContext context) {
        boolean answerToChallenge = getlogonProcessPermission().implies(getPermission(context));
        return answerToChallenge;
    }


    public abstract void buildChallenge(AccessContext context) throws AuthenticationException;

    public void handleSchemeCallbacks(AccessContext context, List<Callback> cbks) throws UnsupportedCallbackException {
        String login = getLogin(context);
        String password = getPassword(context);
        for (Callback cb : cbks) {
            if (cb instanceof NameCallback) {
                ((NameCallback) cb).setName(login);
            } else if (cb instanceof PasswordCallback) {
                ((PasswordCallback) cb).setPassword(password.toCharArray());
            }
        }
    }

    protected abstract String getLogin(AccessContext context);

    protected abstract String getPassword(AccessContext context);

    protected abstract Permission getPermission(AccessContext context);

    protected abstract Permission getlogonProcessPermission();

}
