/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name:  $
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.core.util;

import java.io.BufferedOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.Iterator;
import java.util.List;

import net.sf.jguard.core.principals.RolePrincipal;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.SAXReader;
import org.dom4j.io.XMLWriter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.xml.sax.SAXException;

/**
 * Utility class to handle XML documents with DOM4J.
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 *
 */
public class XMLUtils {

	private static final String UTF_8 = "UTF-8";


	private static Logger logger = LoggerFactory.getLogger(XMLUtils.class.getName());
	
	/**
	* read the xml data storage file for users and associated principals.
	* @param location
	* @return xml content
	*/
	public static Document read(String location) {

		location = resolveLocation(location);
		SAXReader reader = new SAXReader(true);
                Document document = null;
		URL url = null;
             try {
                    //activate schema validation
                    reader.setFeature("http://xml.org/sax/features/validation", true);
                    reader.setFeature("http://apache.org/xml/features/validation/schema", true);
	            url = new URL(location);
	     } catch (SAXException ex) {
                      logger.error("read(String) : "+ex.getMessage(), ex);
                      throw new IllegalArgumentException(ex.getMessage());
             } catch (MalformedURLException e) {
                logger.error("read(String) : " + e);
                if(logger.isDebugEnabled()){
                        logger.debug("read() -  the 'fileLocation' value ( " + location
                                        + " ) present in the web.xml file is a malformed URL ");
                }
                throw new IllegalArgumentException(e.getMessage());
             }
	       
	        try {
	             document = reader.read(url);
	        } catch (DocumentException e1) {
                    logger.error("read(String) : " + e1,e1);
                    throw new IllegalArgumentException(e1.getMessage());
	        }
        
        if(document==null){
        	throw new IllegalArgumentException(location+" does not point to an XML document ");
        }
        return document;
	}

	
        
        public static String resolveLocation(String location) {
		if(location==null){
			throw new IllegalArgumentException(" location is null ");
		}
                
                 //if it does not contains a ':' which is used with a protocol
                //or is the second character like in C: windows path
		if(-1==location.indexOf(':')||1==location.indexOf(':')){
                    URL url = Thread.currentThread().getContextClassLoader().getResource(location);
                    if(url ==null){
                      location = new StringBuffer("file://////").append(location).toString();
                    }else{
                      location = url.toString();
                    }
                }

		//remove trailing space character
		location = location.trim();
                URI uri = null;
        try {

             uri = new URI(null, null, null, -1, location, null, null);
        } catch (URISyntaxException ex) {
            logger.error( "location cannot be converted to an URI path "+ex.getMessage(), ex);
            throw new IllegalArgumentException("location cannot be converted to an URI path "+ex.getMessage());
        }


                if(logger.isDebugEnabled()){
                                logger.debug("location="+location);
                }
		return uri.toString();
	}

	public static void write(String fileLocation,Document document) throws IOException{
                String resolvedLocation = resolveLocation(fileLocation);
		XMLUtils.write(new URL(resolvedLocation),document);
	}

	/**
	 * write the updated configuration to the XML file in the UTF-8 format.
	 * @param url - URL of the file to write
	 * @param document dom4j Document
	 * @throws IOException
	 */
	private static void write(URL url,Document document) throws IOException{
	    OutputFormat outFormat = OutputFormat.createPrettyPrint();
	    if(document.getXMLEncoding()!=null){
	    	outFormat.setEncoding(document.getXMLEncoding());
	    }else{
	    	outFormat.setEncoding(UTF_8);
	    }
	    XMLWriter out = new XMLWriter(new BufferedOutputStream(new FileOutputStream(url.getPath())),outFormat);
	    out.write(document);
	    out.flush();
	    out.close();
	}

	 /**
     * remove in the ascendants elements all the <i>principalRef</i> references with the name of the
     * principal.
     * @param root DOM4J Element of the document
     * @param principal
     */
    public static  void deletePrincipalRefs(Element root, RolePrincipal principal){
    	Element principalsElement = root.element("principals");
        List principalsRefList = principalsElement.selectNodes("//principalRef[@name='"+principal.getLocalName()+"']");
        Iterator descendantsRefIt = principalsRefList.iterator();
        while(descendantsRefIt.hasNext()){
        	Element principalRef = (Element)descendantsRefIt.next();
        	Element principalElement = principalRef.getParent();
        	principalElement.remove(principalRef);
        	if(principalElement.elements("principalRef").size()==0){
        		principalElement.getParent().remove(principalElement);
        	}
        }
    }


}
